

{% load widget_tweaks %}
{% csrf_token %}

<div class="grid grid-cols-6 gap-1">
  {% for field in form %}
    {% component "form_field" field=field%}{% endcomponent %}
  {% endfor %}
</div>

{% for field in form.hidden_fields %}
  {% render_field field %}
{% endfor %}

{% slot "form_footer" %}

{% endslot %}
