{% load django_tables2 i18n %}
{% block table-wrapper %}
  <div class="table-container">
    {% block table %}
      <table {% render_attrs table.attrs %}>
        {% block table.thead %}
          {% if table.show_header %}
            <thead {{ table.attrs.thead.as_html }}>
              <tr>
                {% for column in table.columns %}
                  <th {{ column.attrs.th.as_html }}>
                    {% if column.orderable %}
                      <a href="{% querystring table.prefixed_order_by_field=column.order_by_alias.next %}">{% if "desc" in column.attrs.th.class %}&#8595; {% elif "asc" in column.attrs.th.class %}&#8593; {% endif %}{{ column.header }}</a>
                    {% else %}
                      {{ column.header }}
                    {% endif %}
                  </th>
                {% endfor %}
              </tr>
            </thead>
          {% endif %}
        {% endblock table.thead %}
        {% block table.tbody %}
          <tbody {{ table.attrs.tbody.as_html }}>
            {% for row in table.paginated_rows %}
              {% block table.tbody.row %}
                <tr {{ row.attrs.as_html }}>
                  {% for column, cell in row.items %}
                    <td {{ column.attrs.td.as_html }}>{% if column.localize == None %}{{ cell }}{% else %}{% if column.localize %}{{ cell|localize }}{% else %}{{ cell|unlocalize }}{% endif %}{% endif %}</td>
                  {% endfor %}
                </tr>
              {% endblock table.tbody.row %}
            {% empty %}
              {% if table.empty_text %}
                {% block table.tbody.empty_text %}
                  <tr><td colspan="{{ table.columns|length }}">{{ table.empty_text }}</td></tr>
                {% endblock table.tbody.empty_text %}
              {% endif %}
            {% endfor %}
          </tbody>
        {% endblock table.tbody %}
        {% block table.tfoot %}
          {% if table.has_footer %}
            <tfoot {{ table.attrs.tfoot.as_html }}>
              <tr>
                {% for column in table.columns %}
                  <td {{ column.attrs.tf.as_html }}>{{ column.footer }}</td>
                {% endfor %}
              </tr>
            </tfoot>
          {% endif %}
        {% endblock table.tfoot %}
      </table>
    {% endblock table %}
  </div>
{% endblock table-wrapper %}
