{% extends '../../layouts/index.html' %}

{% block content %}
  <div class="min-h-full  px-4 py-16 sm:px-6 sm:py-24 md:grid md:place-items-center lg:px-8">
    <div class="mx-auto max-w-max">
      <main class="sm:flex">
        <p class="text-4xl font-bold tracking-tight sm:text-5xl">{{ code }}</p>
        <div class="sm:ml-6">
          <div class="sm:border-l sm:border-gray-200 sm:pl-6">
            <h1 class="text-4xl font-bold tracking-tight  sm:text-5xl">{{ title }}</h1>
            <p class="mt-1 text-base-content text-opacity-80">{{ message }}</p>
            {% comment %} <p class="mt-1 text-base text-base-content text-opacity-80">{{ details }}</p> {% endcomment %}
          </div>

          <div class="mt-10 flex space-x-3 sm:border-l sm:border-transparent sm:pl-6">
            <a href="/" class="btn btn-primary">Go back home</a>
          </div>
        </div>
      </main>
    </div>
  </div>
{% endblock content %}
