{% if environment.value is not none %}
  <span class="px-2 inline-flex text-xs leading-5 font-semibold rounded-full
               {% if environment.value == "PRD" %}
                 bg-yellow-100 text-yellow-800
               {% else %}
                 bg-neutral-300 text-neutral-600
               {% endif %}
              ">
    {{ environment.value }}
  </span>
{% endif %}
