# Generated by Django 4.2.21 on 2025-06-02 16:29
import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("incidents", "0011_update_incidents"),
    ]

    operations = [
        migrations.AlterField(
            model_name="impactlevel",
            name="id",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, primary_key=True, serialize=False
            ),
        ),
        migrations.AlterField(
            model_name="impactlevel",
            name="value",
            field=models.CharField(
                choices=[
                    ("HT", "Highest"),
                    ("HI", "High"),
                    ("MD", "Medium"),
                    ("LO", "Low"),
                    ("LT", "Lowest"),
                    ("NO", "N/A"),
                ],
                default="NO",
                max_length=2,
            ),
        ),
        migrations.AlterField(
            model_name="impacttype",
            name="emoji",
            field=models.CharField(default="", max_length=5),
        ),
    ]
