from __future__ import annotations

from django.conf import settings

from firefighter.api.views import (
    components,
    environments,
    groups,
    incident_cost_types,
    incident_costs,
    incidents,
    severities,
)
