# Generated by Django 4.2.11 on 2024-05-03 09:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("raid", "0001_initial_oss"),
    ]

    operations = [
        migrations.CreateModel(
            name="FeatureTeam",
            fields=[
                ("id", models.AutoField(primary_key=True, serialize=False)),
                ("name", models.CharField(max_length=80)),
                ("jira_project_key", models.CharField(max_length=10, unique=True)),
            ],
            options={
                "verbose_name": "Feature Team",
                "verbose_name_plural": "Feature Teams",
                "unique_together": {("name", "jira_project_key")},
            },
        ),
        migrations.RemoveField(
            model_name="qualifierrotation",
            name="jira_user",
        ),
        migrations.DeleteModel(
            name="Qualifier",
        ),
        migrations.DeleteModel(
            name="QualifierRotation",
        ),
    ]
